@@ -894,7 +894,8 @@
 (defoptimizer (make-sequence constraint-propagate-result) ((type length &rest args) node)
   (list (list 'vector-length length)))
 
-(defoptimizers constraint-propagate-result (remove delete remove-if delete-if remove-if-not delete-if-not)
+(defoptimizers constraint-propagate-result (remove delete remove-if delete-if remove-if-not delete-if-not
+                                            copy-remove copy-remove-if-not copy-remove-if)
     ((x sequence &rest args &key from-end test test-not start end count key) node gen)
   (let (c)
     (let ((var (ok-lvar-lambda-var sequence gen)))
@@ -971,6 +972,19 @@
                         (incf max-sum n)
                         (loop repeat n
                               do (pop args))))
+                     ((and subseq
+                           (constant-lvar-p arg)
+                           (eq (lvar-value arg) 'sb-impl::%repeat))
+                      (let* ((length (pop args))
+                             (int (type-approximate-interval (lvar-type length))))
+                        (cond (int
+                               (incf min-sum (or (interval-low int) 0))
+                               (if (interval-high int)
+                                   (incf max-sum (interval-high int))
+                                   (setf max nil)))
+                              (t
+                               (setf max nil))))
+                      (pop args))
                      (t
                       (let ((int (type-approximate-interval (type-intersection (or (vector-length-type (lvar-type arg))
                                                                                    *universal-type*)
