@@ -24,7 +24,8 @@
 (define-source-transform char-int (x)
   `(char-code ,x))
 
-(deftransform abs ((x) (rational))
+(deftransform abs ((x) (rational) * :node node)
+  (delay-ir1-transform node :constraint)
   '(if (< x 0) (- x) x))
 
 (deftransform make-symbol ((string) (simple-string))
@@ -444,7 +445,8 @@
                                 (specifier-type '(simple-array * (*))))
                finally (return (sb-kernel::%type-union types)))))
       (t
-       (when final
+       (when (or final
+                 (csubtypep type (specifier-type 'simple-array)))
          (specifier-type '(simple-array * (*))))))))
 
 (defoptimizer (array-storage-vector derive-type) ((array))
@@ -484,8 +486,8 @@
     (if (array-header-p array)
         (values (%array-data array) index)
         (values array index))
-    (%check-bound array (length data) index)
-    (values array index)))
+    (%check-bound array (vector-length data) index)
+    (values data index)))
 
 ;;; Only use %data-vector-and-index if element-type is known. Always
 ;;; using %data-vector-and-index will not help if a call to
