@@ -505,14 +505,13 @@ body, references to a NAME will effectively be replaced with the EXPANSION."
                nargs
                min)))
 
-    (when (template-conditional-p template)
-      (bug "%PRIMITIVE was used with a conditional template."))
-
     (when (template-more-results-type template)
       (bug "%PRIMITIVE was used with an unknown values template."))
 
     (ir1-convert start next result
-                 `(%%primitive ',template ,@args))))
+                 (if (template-conditional-p template)
+                     `(if (%%primitive ',template ,@args) t nil)
+                     `(%%primitive ',template ,@args)))))
 
 (defmacro inline-%primitive (template &rest args)
   (let* ((required (length (template-arg-types template)))
@@ -1296,8 +1295,9 @@ care."
         (dest-lvar (make-lvar))
         (type (or (lexenv-find var type-restrictions)
                   (leaf-type var))))
-    (ir1-convert start dest-ctran dest-lvar `(the ,(type-specifier type)
-                                                  ,value))
+    (ir1-convert start dest-ctran dest-lvar (wrap-if (neq type *universal-type*)
+                                                     `(the ,(type-specifier type))
+                                                     value))
     (let ((res (make-set var dest-lvar)))
       (setf (lvar-dest dest-lvar) res)
       (cond (result ; SETQ with a result counts as a REF also
