@@ -1023,18 +1023,34 @@ specify bindings for printer control variables.")
 (defmacro make-defs (vars &body body)
   (labels ((subst-if-with (test tree)
              (labels ((s (subtree)
-                        (let ((test (funcall test subtree)))
-                          (cond (test)
+                        (multiple-value-bind (new replace)
+                            (funcall test subtree)
+                          (cond (replace new)
                                 ((atom subtree) subtree)
                                 (t (let ((car (s (car subtree)))
                                          (cdr (s (cdr subtree))))
                                      (if (and (eq car (car subtree))
                                               (eq cdr (cdr subtree)))
                                          subtree
-                                         (if (typep car '(cons (eql $when)))
-                                             (and (eval (second car))
-                                                  (cddr car))
-                                             (cons car cdr)))))))))
+                                         (cond ((and (typep car '(cons symbol))
+                                                     (string= (car car) "$WHEN"))
+                                                (if (eval (second car))
+                                                    (append (cddr car) cdr)
+                                                    cdr))
+                                               ((and (typep car '(cons symbol))
+                                                     (string= (car car) "$UNLESS"))
+                                                (if (eval (second car))
+                                                    cdr
+                                                    (append (cddr car) cdr)))
+                                               ((and (typep car '(cons symbol))
+                                                     (string= (car car) "$IF"))
+                                                (cons
+                                                 (if (eval (second car))
+                                                     (third car)
+                                                     (fourth car))
+                                                 cdr))
+                                               (t
+                                                (cons car cdr))))))))))
                (s tree)))
            (test (pattern with)
              (lambda (x)
@@ -1043,10 +1059,11 @@ specify bindings for printer control variables.")
                         (start (search pattern str)))
                    (when start
                      (if (equal str pattern)
-                         with
-                         (symbolicate (subseq str 0 start)
-                                      with
-                                      (subseq str (+ start (length pattern))))))))))
+                         (values with t)
+                         (values (symbolicate (subseq str 0 start)
+                                              with
+                                              (subseq str (+ start (length pattern))))
+                                 t)))))))
            (gen (vars body)
              (if vars
                  (loop for with in (cdar vars)
