@@ -124,7 +124,10 @@
              ((nil) 'unsigned-byte)
              ((t) 'signed-byte))
           ,width)
-         (foldable flushable movable always-translatable)
+         (foldable flushable movable always-translatable
+                   ,@(and (ir1-attributep (fun-info-attributes (fun-info-or-lose prototype))
+                                          commutative)
+                          '(commutative)))
        :derive-type (make-modular-fun-type-deriver ',prototype ,width ',signedp))))
 
 (defun %define-good-modular-fun (name kind signedp)
@@ -314,17 +317,22 @@
                             (specifier-type 'integer))
                     (let (did-something)
                       (do-uses (combination (cast-value node))
-                        (when (and (combination-matches* '(+ -) '(* *) combination)
+                        (when (and (or (combination-matches* '(+ -) '(* *) combination)
+                                       (combination-matches* '(%negate) '(*) combination))
                                    (almost-immediately-used-p (node-lvar combination) combination
                                                               :flushable t))
-                          (destructuring-bind (a b) (combination-args combination)
+                          (destructuring-bind (a &optional b) (combination-args combination)
                             (when (or (not (types-equal-or-intersect (lvar-type a)
                                                                      #1=(specifier-type '(or ratio (complex rational)))))
-                                      (not (types-equal-or-intersect (lvar-type b) #1#)))
+                                      (not (and b
+                                                (types-equal-or-intersect (lvar-type b) #1#))))
                               (when (cut-node combination)
                                 (setf did-something t))))))
                       (when did-something
-                        (change-return-type node (values-specifier-type '(values integer &optional))))
+                        (change-return-type node
+                                            (if (type-single-value-p (node-derived-type node))
+                                                (values-specifier-type '(values integer &optional))
+                                                (values-specifier-type '(values integer)))))
                       nil)))))
              (cut-lvar (lvar &key head
                         &aux did-something must-insert over-wide)
@@ -387,8 +395,9 @@
           (return-from best-modular-version
             (values (car ugt) :untagged (cdr ugt))))))))
 
-(defoptimizer (logand optimizer) ((x y) node)
-  (let ((result-type (single-value-type (node-derived-type node))))
+(defoptimizer (logand optimizer) ((x y) node &optional result-type)
+  (let ((result-type (or result-type
+                         (single-value-type (node-derived-type node)))))
     (multiple-value-bind (low high)
         (integer-type-numeric-bounds result-type)
       (when (and (numberp low)
@@ -419,6 +428,11 @@
               )))))))
 
 (setf (fun-info-optimizer (fun-info-or-lose 'logandc2)) #'logand-optimizer-optimizer)
+(setf (fun-info-optimizer (fun-info-or-lose 'logtest))
+      (lambda (node)
+        (let ((type (logand-derive-type-optimizer node)))
+          (when type
+           (logand-optimizer-optimizer node type)))))
 
 (defoptimizer (mask-signed-field optimizer) ((width x) node)
   (let ((result-type (single-value-type (node-derived-type node))))
@@ -502,6 +516,141 @@
               `(lambda (value amount1 amount2)
                  (logand (ash value (+ amount1 amount2))
                          ,(1- (ash 1 (+ width (lvar-value amount))))))))))))
+
+(deftransform ash-into-word-mod ((x count) (t (constant-arg (integer #.(- sb-vm:n-word-bits) 0))))
+  `(typecase x
+     (fixnum
+      (logand most-positive-word (ash (truly-the fixnum x) count)))
+     (sb-vm:signed-word
+      (logand most-positive-word (ash (truly-the sb-vm:signed-word x) count)))
+     (t
+      (ash-right-two-words (sb-bignum:%bignum-ref (truly-the bignum x) 1)
+                           (sb-bignum:%bignum-ref (truly-the bignum x) 0)
+                           (- count)))))
+
+(deftransform ash-into-word-mod ((x count) * * :important nil :priority :last)
+  (let* ((minusp (csubtypep (lvar-type count) (specifier-type '(integer * 0))))
+         (right
+           `(typecase x
+              (fixnum
+               (logand most-positive-word (ash (truly-the fixnum x) ,(if minusp
+                                                                         `(- count)
+                                                                         `count))))
+              (bignum
+               (multiple-value-bind (words bits) (truncate ,(if minusp
+                                                                `count
+                                                                `(- count))
+                                                           sb-vm:n-word-bits)
+                 (let ((length (%bignum-length (truly-the bignum x))))
+                   (flet ((extend-ref (index)
+                            (if (< index length)
+                                (sb-bignum:%bignum-ref (truly-the bignum x) (truly-the bignum-index index))
+                                (sb-bignum::%sign-digit (truly-the bignum x) length))))
+                     (declare (inline extend-ref))
+                     (ash-right-two-words (extend-ref (1+ words))
+                                          (extend-ref words)
+                                          (truly-the (mod ,sb-vm:n-word-bits) bits)))))))))
+    (if minusp
+        `(let ((count (- count)))
+           ,right)
+        `(if (minusp count)
+             ,right
+             (logand most-positive-word (ash x (truly-the unsigned-byte count)))))))
+
+(defun ash-into-word-mod (x count)
+  (logand (ash x count) most-positive-word))
+
+(defun ash-into-word-modfx (x count)
+  (mask-signed-field sb-vm:n-fixnum-bits (ash x count)))
+
+(deftransform ash-into-word-mod ((x count) (t (constant-arg (integer * (#.(- sb-vm:n-word-bits))))))
+  (let ((count (- (lvar-value count))))
+    (multiple-value-bind (words bits) (truncate count sb-vm:n-word-bits)
+      `(typecase x
+         (sb-vm:signed-word
+          (logand most-positive-word (ash (truly-the sb-vm:signed-word x) count)))
+         (t
+          (let ((length (%bignum-length (truly-the bignum x))))
+            (flet ((extend-ref (index)
+                     (if (< index length)
+                         (sb-bignum:%bignum-ref (truly-the bignum x) index)
+                         (sb-bignum::%sign-digit (truly-the bignum x) length))))
+              (declare (inline extend-ref))
+              ,(if (zerop bits)
+                   `(extend-ref ,words)
+                   `(ash-right-two-words (extend-ref ,(1+ words))
+                                         (extend-ref ,words)
+                                         ,bits)))))))))
+
+(deftransform ash-into-word-modfx ((x count) (t (constant-arg (integer #.(- sb-vm:n-word-bits) 0))))
+  `(typecase x
+     (fixnum
+      (mask-signed-field sb-vm:n-fixnum-bits
+                         (ash (truly-the fixnum x) count)))
+     (sb-vm:signed-word
+      (mask-signed-field sb-vm:n-fixnum-bits
+                         (ash (truly-the sb-vm:signed-word x) count)))
+     (t
+      (mask-signed-field sb-vm:n-fixnum-bits
+                         (ash-right-two-words (sb-bignum:%bignum-ref (truly-the bignum x) 1)
+                                              (sb-bignum:%bignum-ref (truly-the bignum x) 0)
+                                              (- count))))))
+
+(deftransform ash-into-word-modfx ((x count) * * :important nil :priority :last)
+  (let* ((minusp (csubtypep (lvar-type count) (specifier-type '(integer * 0))))
+         (right
+           `(typecase x
+              (fixnum
+               (mask-signed-field sb-vm:n-fixnum-bits
+                                  (ash (truly-the fixnum x) ,(if minusp
+                                                                 `(- count)
+                                                                 `count))))
+              (t
+               (multiple-value-bind (words bits) (truncate ,(if minusp
+                                                                `count
+                                                                `(- count))
+                                                           sb-vm:n-word-bits)
+                 (let ((length (%bignum-length (truly-the bignum x))))
+                   (flet ((extend-ref (index)
+                            (if (< index length)
+                                (sb-bignum:%bignum-ref (truly-the bignum x) (truly-the bignum-index index))
+                                (sb-bignum::%sign-digit (truly-the bignum x) length))))
+                     (declare (inline extend-ref))
+                     (mask-signed-field sb-vm:n-fixnum-bits
+                                        (ash-right-two-words (extend-ref (1+ words))
+                                                             (extend-ref words)
+                                                             bits)))))))))
+    (if minusp
+        `(let ((count (- count)))
+           ,right)
+        `(if (minusp count)
+             ,right
+             (logand most-positive-word (ash x (truly-the unsigned-byte count)))))))
+
+(deftransform ash-into-word-modfx ((x count) (t (constant-arg (integer * (#.(- sb-vm:n-word-bits))))))
+  (let ((count (- (lvar-value count))))
+    (multiple-value-bind (words bits) (truncate count sb-vm:n-word-bits)
+      `(typecase x
+         (sb-vm:signed-word
+          (mask-signed-field sb-vm:n-fixnum-bits
+                             (ash (truly-the sb-vm:signed-word x) count)))
+         (t
+          (let ((length (%bignum-length (truly-the bignum x))))
+            (flet ((extend-ref (index)
+                     (if (< index length)
+                         (sb-bignum:%bignum-ref (truly-the bignum x) index)
+                         (sb-bignum::%sign-digit x length))))
+              (declare (inline extend-ref))
+              (mask-signed-field sb-vm:n-fixnum-bits
+                                 ,(if (zerop bits)
+                                      `(extend-ref ,words)
+                                      `(ash-right-two-words (extend-ref ,(1+ words))
+                                                            (extend-ref ,words)
+                                                            ,bits))))))))))
+
+(deftransform ash-right-two-words ((w2 w1 count) (t t (eql #.sb-vm:n-word-bits)) * :important nil)
+  'w2)
+
 (macrolet
     ((def (left-name name kind width signedp)
        (declare (ignorable name))
@@ -533,10 +682,16 @@
                                   (csubtypep integer-type (specifier-type `(signed-byte ,sb-vm:n-word-bits)))))
                          ',name)
                         ((and (not (csubtypep integer-type (specifier-type 'word)))
-                              (not (csubtypep integer-type (specifier-type 'sb-vm:signed-word)))
-                              (csubtypep count-type (specifier-type `(integer ,(- result-width width) ,most-positive-fixnum))))
-                         (cut-to-width integer ,kind width ,signedp)
-                         t)))))
+                              (not (csubtypep integer-type (specifier-type 'sb-vm:signed-word))))
+                         (cond ((csubtypep count-type (specifier-type `(integer ,(- result-width width) ,most-positive-fixnum)))
+                                (cut-to-width integer ,kind width ,signedp)
+                                t)
+                               #+(or arm64 x86-64)
+                               ((and (csubtypep count-type (specifier-type 'fixnum))
+                                     (not (csubtypep count-type (specifier-type 'unsigned-byte))))
+                                ',(if signedp
+                                      'ash-into-word-modfx
+                                      'ash-into-word-mod))))))))
             (setf (gethash ',left-name (modular-class-versions (find-modular-class ',kind ',signedp)))
                   `(ash ,',width))
             (deftransform ,left-name ((integer count) (t (constant-arg (eql 0))))
