@@ -519,7 +519,10 @@
   (xrefs nil :type list)
   ;; Cache the environment of a block during lifetime analysis. :NONE
   ;; if no cached value has been stored yet.
-  (environment-cache :none :type (or null environment (member :none))))
+  (environment-cache :none :type (or null environment (member :none)))
+  ;; A table for keeping track of which source-paths have already had
+  ;; a %MARK-COVERAGE function converted for them.
+  (source-path-marks nil :type (or null hash-table)))
 (defmethod print-object ((cblock cblock) stream)
   (if (boundp '*compilation*)
       (print-unreadable-object (cblock stream :type t :identity t)
@@ -650,8 +653,7 @@
   (delete-blocks nil :type list)
   ;; The default LOOP in the component.
   (outer-loop (missing-arg) :type cloop)
-  (max-block-number 0 :type fixnum)
-  (dominators-computed nil))
+  (renumber-p nil))
 
 (defprinter (component :identity t)
   name
@@ -1441,11 +1443,12 @@
   explicit-value-cell
   ;; Do not propagate constraints for this var
   no-constraints
-  ;; Does it hold a constant that should't be destructively modified
+  ;; Does it hold a constant that shouldn't be destructively modified
   constant
   unused-initial-value
   ;; Instruct constraint propagation to do some work.
-  compute-same-refs)
+  compute-same-refs
+  no-debug)
 
 (defstruct (lambda-var
             (:include basic-var) (:copier nil)
@@ -1503,7 +1506,8 @@
   `(lambda-var-attributep (lambda-var-flags ,var) constant))
 (defmacro lambda-var-unused-initial-value (var)
   `(lambda-var-attributep (lambda-var-flags ,var) unused-initial-value))
-
+(defmacro lambda-var-no-debug (var)
+  `(lambda-var-attributep (lambda-var-flags ,var) no-debug))
 (defmacro lambda-var-compute-same-refs (var)
   `(lambda-var-attributep (lambda-var-flags ,var) compute-same-refs))
 
@@ -1530,7 +1534,10 @@
   (same-refs nil :type (or null cons)))
 (defprinter (ref :identity t)
   (%source-name :test (neq %source-name '.anonymous.))
-  leaf)
+  (leaf :prin1 (if (and (constant-p leaf)
+                        (not (symbolp (constant-value leaf))))
+                   (constant-value leaf)
+                   leaf)))
 
 (defstruct (multiple-successors-node
             (:constructor nil)
@@ -1639,7 +1646,10 @@
   (pass-nargs t :type boolean)
   (or-chain-computed nil :type boolean))
 (defprinter (combination :identity t)
-  (fun :prin1 (lvar-uses fun))
+  (fun :prin1 (let ((uses (lvar-uses fun)))
+                (or (and (ref-p uses)
+                         (ref-fun-name uses t))
+                    uses)))
   (args :prin1 (mapcar (lambda (x)
                          (if x
                              (lvar-uses x)
@@ -1792,7 +1802,11 @@
   (funs nil :type list)
   ;; The dynamic extent for this enclose if any of its functionals are
   ;; declared dynamic extent.
-  (dynamic-extent nil :type (or null cdynamic-extent)))
+  (dynamic-extent nil :type (or null cdynamic-extent))
+  ;; The union of all dynamic extents inferred by the compiler for
+  ;; this enclose's functionals. NULL when there is a declared dynamic
+  ;; extent.
+  (derived-dynamic-extents () :type list))
 (defprinter (enclose :identity t)
   funs)
 
@@ -1806,12 +1820,10 @@
                             (:copier nil))
   ;; the values explicitly declared with this dynamic extent.
   (values nil :type list)
-  ;; the cleanup for this extent. NULL indicates that this dynamic
-  ;; extent is over the environment and hence needs no cleanup code.
+  ;; the cleanup for this extent.
   (cleanup nil :type (or cleanup null))
   ;; some kind of info used by the back end
-  (info nil)
-  (preserve-info nil))
+  (info nil))
 
 (defprinter (cdynamic-extent :conc-name dynamic-extent-
                              :identity t)
