@@ -47,7 +47,7 @@
                 (or (eq name 'make-array)
                     (equal name '(setf %array-fill-pointer))))
            (when (eq name 'make-array) ; nullify the creating PC location
-             (inst mov :qword (object-slot-ea object 1 lowtag) nil-value))
+             (inst mov :qword (object-slot-ea object 1 lowtag) null-tn))
            (inst mov :dword (vector-len-ea object)
                  (or (encode-value-if-immediate value) value)))
           #-soft-card-marks
@@ -63,13 +63,6 @@
            (when barrier (emit-gengc-barrier object nil val-temp t))
            (emit-store (object-slot-ea object offset lowtag) value val-temp)))))
 
-(defun add-symbol-to-remset (vop symbol)
-  (declare (ignorable vop symbol))
-  #+permgen
-  (unless (and (sc-is symbol immediate) (static-symbol-p (tn-value symbol)))
-    (inst push symbol)
-    (invoke-asm-routine 'call 'gc-remember-symbol vop)))
-
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg) :to :eval)
          (old :scs (descriptor-reg any-reg) #|:target rax|#)
@@ -83,9 +76,10 @@
   (:results (result :scs (descriptor-reg any-reg)))
   (:vop-var vop)
   (:generator 5
-     (when (member name '(cas-symbol-fdefn sb-impl::cas-symbol-%info))
-       (add-symbol-to-remset vop object))
-     (emit-gengc-barrier object nil rax (vop-nth-arg 2 vop))
+     (let ((newval-tn-ref (vop-nth-arg 2 vop)))
+       (if (eq name 'sb-impl::cas-symbol-%info)
+           (emit-symbol-write-barrier vop object rax newval-tn-ref)
+           (emit-gengc-barrier object nil rax newval-tn-ref)))
      (move rax old)
      (inst cmpxchg :lock (ea (- (* offset n-word-bytes) lowtag) object) new)
      (move result rax)))
@@ -96,11 +90,15 @@
   (:args)
   (:results (result :scs (descriptor-reg any-reg)))
   (:generator 1
-    (inst mov result (unbound-marker-bits))))
+    (inst mov result unbound-marker-widetag)))
 
 (defun emit-symbol-write-barrier (vop symbol temp newval-tn-ref)
+  (declare (ignorable vop))
+  #+permgen
   (when (require-gengc-barrier-p symbol newval-tn-ref)
-    (add-symbol-to-remset vop symbol))
+    (unless (and (sc-is symbol immediate) (static-symbol-p (tn-value symbol)))
+      (inst push symbol)
+      (invoke-asm-routine 'call 'gc-remember-symbol vop)))
   ;; IMMEDIATE sc means that the symbol is static or immobile.
   ;; Static symbols are roots, and immobile symbols use page fault handling.
   (unless (sc-is symbol immediate)
@@ -108,11 +106,11 @@
 
 ;; Return the effective address of the value slot of static SYMBOL.
 (defun static-symbol-value-ea (symbol &optional (byte 0))
-   (ea (+ nil-value
-          (static-symbol-offset symbol)
-          (ash symbol-value-slot word-shift)
-          byte
-          (- other-pointer-lowtag))))
+  (ea (+ (static-symbol-offset symbol)
+         (ash symbol-value-slot word-shift)
+         byte
+         (- other-pointer-lowtag))
+      null-tn))
 
 (define-vop (fast-symbol-global-value)
   (:args (object :scs (descriptor-reg immediate)))
@@ -151,12 +149,6 @@
                       (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))
       value val-temp)))
 
-(defun emit-lea-symbol-value-slot (ea-tn symbol) ; dest, src
-  (if (sc-is symbol immediate)
-      (inst mov ea-tn (make-fixup (tn-value symbol) :immobile-symbol
-                                  (- (ash symbol-value-slot word-shift) other-pointer-lowtag)))
-      (inst lea ea-tn (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))))
-
 #-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
@@ -187,7 +179,7 @@
     (:generator 15
       (emit-symbol-write-barrier vop symbol rax (vop-nth-arg 2 vop))
       (if (sc-is old immediate)
-          (inst mov rax (encode-value-if-immediate old))
+          (move-immediate rax (immediate-tn-repr old))
           (move rax old))
       (inst cmpxchg :lock (object-slot-ea symbol symbol-value-slot other-pointer-lowtag) new)
       (unless (policy node (= safety 0))
@@ -222,6 +214,13 @@
       (inst sub bsp (* binding-size n-word-bytes))
       (store-binding-stack-pointer bsp))))
 
+;;; I don't know which of SYMBOL-HASH or SYMBOL-NAME-HASH is dynamically executed most.
+;;; Whichever one that is can slightly optimized more by avoiding a load from the hash of NIL
+;;; and using NULL-TN directly. Currently this favors SYMBOL-HASH.
+;;; To favor SYMBOL-NAME-HASH, we would need to flip the position of the hash and fname-index
+;;; fields within the hash slot.  Putting HASH in the low 4 bytes allows a :DWORD XOR with
+;;; NULL-TN to achieve position-independence of NIL's hash.
+;;; Such minutia I do not care to deal with at the moment.
 (define-vop (symbol-hash)
   (:policy :fast-safe)
   (:translate symbol-hash)
@@ -230,6 +229,7 @@
   (:result-types positive-fixnum)
   (:generator 2
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
+    (inst xor res null-tn)
     (inst shr res n-symbol-hash-discard-bits)))
 
 (eval-when (:compile-toplevel)
@@ -244,10 +244,16 @@
   ;; identical translations believe it or not
   (:translate symbol-name-hash hash-as-if-symbol-name)
   (:generator 1
-    ;; NIL gets 0 for its name hash since its upper 4 address bytes are 0
+    ;; The number of times this vop is executed on an arg _not_ known to be a symbol
+    ;; is about 20x the number of times it _is_ known to be a symbol. (Think of all the
+    ;; CASE expressions where the arg type was not pre-tested)
+    ;; It is quick to unconditionally XOR with a word that is likely in L1 cache already
+    ;; (CAR of NIL) versus conditionally branching, making this vop more efficient than
+    ;; with the alternate definition of SYMBOL slots which put HASH near the end.
     (inst mov :dword res
           (ea (- (+ 4 (ash symbol-hash-slot word-shift)) other-pointer-lowtag)
-              symbol))))
+              symbol))
+    (inst xor :dword res (ea (- 4 list-pointer-lowtag) null-tn))))
 
 (aver (= sb-impl::package-id-bits 16))
 (define-vop (symbol-package-id)
@@ -311,7 +317,7 @@
   (:generator 2
     (loadw result fdefn fdefn-fun-slot other-pointer-lowtag)
     (inst test :dword result result)
-    (inst cmov :z result (ea (- nil-value list-pointer-lowtag)))))
+    (inst cmov :z result null-tn)))
 
 (define-vop (safe-fdefn-fun)
   (:translate safe-fdefn-fun)
@@ -534,20 +540,18 @@
       (when use-xmm-p
         (inst xorpd xmm-temp xmm-temp))
       (loop
-       (let* ((slot (pop indices))
+       (let* ((index (pop indices))
               (val (tn-ref-tn values))
-              (ea (ea (- (ash (+ instance-slots-offset slot) word-shift)
-                         instance-pointer-lowtag)
-                      instance)))
-         (aver (tn-p instance))
+              (ea (object-slot-ea instance (+ instance-slots-offset index)
+                                  instance-pointer-lowtag)))
          (setq values (tn-ref-across values))
          ;; If the xmm temp was loaded with 0 and this value is 0,
          ;; and possibly the next, then store through the temp
          (cond
            ((and use-xmm-p (constant-tn-p val) (eql (tn-value val) 0))
-            (let* ((next-slot (car indices))
-                   (next-val (if next-slot (tn-ref-tn values))))
-              (cond ((and (eql (1+ slot) next-slot)
+            (let* ((next-index (car indices))
+                   (next-val (if next-index (tn-ref-tn values))))
+              (cond ((and (eql (1+ index) next-index)
                           (constant-tn-p next-val)
                           (eql (tn-value next-val) 0))
                      (inst movupd ea xmm-temp)
@@ -586,13 +590,12 @@
   (:generator 3
     ;; Use RESULT as the source of the exchange, unless doing so
     ;; would clobber NEWVAL
-    (let ((source (if (location= result instance) temp result)))
+    (let ((source (if (location= result instance) temp result))
+          (slot (+ instance-slots-offset index)))
       (if (sc-is newval immediate)
           (inst mov source (constantize (tn-value newval)))
           (move source newval))
-      (inst xchg (ea (- (ash (+ instance-slots-offset index) word-shift)
-                        instance-pointer-lowtag) instance)
-            source)
+      (inst xchg (object-slot-ea instance slot instance-pointer-lowtag) source)
       (unless (eq source result)
         (move result temp)))))
 
@@ -642,15 +645,38 @@
               (:arg-types * tagged-num)
               (:results (value :scs (,result-sc)))
               (:result-types ,result-type)
-              (:generator 1 (inst ,inst value (instance-slot-ea object index))))
+              (:generator 1
+                (inst ,inst value (instance-slot-ea object index))))
             (define-vop ()
               (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
               (:policy :fast-safe)
               (:args (object :scs (descriptor-reg))
                      (index :scs (any-reg immediate))
-                     (value :scs (,result-sc)))
+                     (value :scs (,result-sc ,@(case result-sc
+                                                (single-reg
+                                                 '(fp-single-immediate fp-single-zero))
+                                                (double-reg
+                                                 '(fp-double-zero))
+                                                (complex-single-reg
+                                                 '(fp-complex-single-zero))
+                                                ((unsigned-reg signed-reg)
+                                                 '((immediate (plausible-signed-imm32-operand-p (tn-value tn)))))))))
               (:arg-types * tagged-num ,result-type)
-              (:generator 1 (inst ,inst (instance-slot-ea object index) value))))))
+              (:generator 1
+                ,(if (eq result-sc 'complex-double-reg)
+                     `(inst ,inst (instance-slot-ea object index) value)
+                     `(sc-case value
+                        (,result-sc
+                         (inst ,inst (instance-slot-ea object index) value))
+                        (t
+                         (inst mov ,(case result-sc
+                                      (single-reg :dword)
+                                      (t :qword))
+                               (instance-slot-ea object index)
+                               ,(case result-sc
+                                  (single-reg '(single-float-bits (tn-value value)))
+                                  ((double-reg complex-single-reg) 0)
+                                  (t '(tn-value value))))))))))))
     (def word unsigned-reg unsigned-num mov)
     (def signed-word signed-reg signed-num mov)
     (def single single-reg single-float movss)
