@@ -205,7 +205,13 @@ Except see also BREAK-VICIOUS-METACIRCLE.  -- CSR, 2003-05-28
 (defun standard-slot-value (object slot-name class)
   (declare (notinline standard-instance-access
                       funcallable-standard-instance-access))
-  (let ((location (gethash (cons class slot-name) *standard-slot-locations*)))
+  ;; I'm sure there's a super easy way to feed the mix of the CLASS and SLOT-NAME
+  ;; hashes into a perfect hash fun, but this function seems never to be called except
+  ;; by MOP some tests. Therefore I don't care to improve it beyond the avoidance
+  ;; of 1 cons operation.
+  (let* ((key (cons class slot-name))
+         (location (gethash key *standard-slot-locations*)))
+    (declare (dynamic-extent key))
     (if location
         (let ((value (if (funcallable-instance-p object)
                          (funcallable-standard-instance-access object location)
@@ -1650,10 +1656,12 @@ Except see also BREAK-VICIOUS-METACIRCLE.  -- CSR, 2003-05-28
           (call-no-applicable-method gf args)))
       (let* ((key (car methods))
              (cache
-              (if (listp key)           ; early method
-                  (sixth key)           ; See !EARLY-MAKE-A-METHOD
-                  (or (method-em-cache key)
-                      (setf (method-em-cache key) (cons nil nil))))))
+               (if (listp key)          ; early method
+                   (sixth key)          ; See !EARLY-MAKE-A-METHOD
+                   (or (method-em-cache key)
+                       (setf (method-em-cache key)
+                             (sb-thread:barrier (:write)
+                               (cons nil nil)))))))
         (if (and (null (cdr methods)) all-applicable-p ; the most common case
                  (null method-alist-p) wrappers-p (not function-p))
             (or (car cache)
@@ -1668,7 +1676,9 @@ Except see also BREAK-VICIOUS-METACIRCLE.  -- CSR, 2003-05-28
                   (let ((value (get-secondary-dispatch-function2
                                 gf methods types method-alist-p wrappers-p
                                 all-applicable-p all-sorted-p function-p)))
-                    (push (cons akey value) (cdr cache))
+                    (setf (cdr cache)
+                          (sb-thread:barrier (:write)
+                            (cons (cons akey value) (cdr cache))))
                     value)))))))
 
 (defun get-secondary-dispatch-function2 (gf methods types method-alist-p
