@@ -55,30 +55,16 @@
 (defun make-dfun-lambda-list (nargs applyp)
   (let ((required (make-dfun-required-args nargs)))
     (if applyp
-        (nconc required
-               ;; Use &MORE arguments to avoid consing up an &REST list
-               ;; that we might not need at all. See MAKE-EMF-CALL and
-               ;; INVOKE-EFFECTIVE-METHOD-FUNCTION for the other
-               ;; pieces.
-               '(&more .dfun-more-context. .dfun-more-count.))
+        (nconc required '(&rest .rest.))
         required)))
 
 (defun make-dlap-lambda-list (nargs applyp)
   (let ((required (make-dfun-required-args nargs)))
-    ;; Return the full lambda list, the required arguments, a form
-    ;; that will generate a rest-list, and a list of the &MORE
-    ;; parameters used.
-    ;; Beware of deep voodoo! The DEFKNOWN for %LISTIFY-REST-ARGS says that its
-    ;; second argument is INDEX, but the THE form below is "weaker" on account
-    ;; of the vop operand restrictions or something that I don't understand.
-    ;; Which is to say, PCL compilation reliably broke when changed to INDEX.
     (if applyp
-        (values (sys-tlab-append required '(&more .more-context. .more-count.))
+        (values (sys-tlab-append required '(&rest .rest.))
                 required
-                '((sb-c:%listify-rest-args
-                   .more-context. (the (and unsigned-byte fixnum)
-                                    .more-count.)))
-                '(.more-context. .more-count.))
+                '((sb-c::%rest-list .rest.))
+                '(.rest.))
         (values required required nil nil))))
 
 (defun make-emf-call (nargs applyp fn-variable &optional emf-type)
@@ -93,14 +79,10 @@
        ;; the :REST-ARG version or the :MORE-ARG version depending on
        ;; the type of the EMF.
        :rest-arg ,(if applyp
-                      ;; Creates a list from the &MORE arguments.
-                      '((sb-c:%listify-rest-args ; See above re. voodoo
-                         .dfun-more-context.
-                         (the (and unsigned-byte fixnum)
-                           .dfun-more-count.)))
+                      '((sb-c::%rest-list .rest.))
                       nil)
        :more-arg ,(when applyp
-                    '(.dfun-more-context. .dfun-more-count.)))))
+                    '(.rest.)))))
 
 (defun make-fast-method-call-lambda-list (nargs applyp)
   (list* '.pv. '.next-method-call. (make-dfun-lambda-list nargs applyp)))
@@ -175,8 +157,16 @@
 
 ;;; FIXME: What do these variables mean?
 (defvar *precompiling-lap* nil)
+(defvar *emit-function-p* t)
+
+;;; Should PCL call compile at runtime to optimize cache functions?
+(defvar *optimize-cache-functions-p* t)
 
 (defun emit-default-only (metatypes applyp)
+  (unless *optimize-cache-functions-p*
+    (when (and (null *precompiling-lap*) *emit-function-p*)
+      (return-from emit-default-only
+        (emit-default-only-function metatypes applyp))))
   (multiple-value-bind (lambda-list args rest-arg more-arg)
       (make-dlap-lambda-list (length metatypes) applyp)
     (generating-lisp '(emf)
@@ -283,7 +273,8 @@
       (:writer `(setf ,read-form ,(car arglist))))))
 
 (defmacro emit-reader/writer-macro (reader/writer 1-or-2-class class-slot-p)
-  (let ((*precompiling-lap* t))
+  (let ((*emit-function-p* nil)
+        (*precompiling-lap* t))
     (values
      (emit-reader/writer reader/writer 1-or-2-class class-slot-p))))
 
@@ -293,6 +284,11 @@
 (defun emit-one-or-n-index-reader/writer (reader/writer
                                           cached-index-p
                                           class-slot-p)
+  (unless *optimize-cache-functions-p*
+    (when (and (null *precompiling-lap*) *emit-function-p*)
+      (return-from emit-one-or-n-index-reader/writer
+        (emit-one-or-n-index-reader/writer-function
+         reader/writer cached-index-p class-slot-p))))
   (multiple-value-bind (arglist metatypes)
       (ecase reader/writer
         ((:reader :boundp :makunbound)
@@ -314,18 +310,15 @@
 
 (defmacro emit-one-or-n-index-reader/writer-macro
     (reader/writer cached-index-p class-slot-p)
-  (let ((*precompiling-lap* t))
+  (let ((*emit-function-p* nil)
+        (*precompiling-lap* t))
     (values
-     (emit-one-or-n-index-reader/writer reader/writer
-                                        cached-index-p
+     (emit-one-or-n-index-reader/writer reader/writer cached-index-p
                                         class-slot-p))))
 
 (defun emit-miss (miss-fn args applyp)
   (if applyp
-      `(multiple-value-call ,miss-fn ,@args
-                            (sb-c:%more-arg-values .more-context.
-                                                    0
-                                                    .more-count.))
+      `(apply ,miss-fn ,@args .rest.)
       `(funcall ,miss-fn ,@args)))
 
 ;; (cache-emf, return-value):
@@ -337,6 +330,11 @@
 ;;  METATYPES must be acceptable to EMIT-FETCH-WRAPPER.
 ;;  APPLYP says whether there is a &MORE context.
 (defun emit-checking-or-caching (cached-emf-p return-value-p metatypes applyp)
+  (unless *optimize-cache-functions-p*
+    (when (and (null *precompiling-lap*) *emit-function-p*)
+      (return-from emit-checking-or-caching
+        (emit-checking-or-caching-function
+         cached-emf-p return-value-p metatypes applyp))))
   (multiple-value-bind (lambda-list args rest-arg more-arg)
       (make-dlap-lambda-list (length metatypes) applyp)
     (generating-lisp
