@@ -430,12 +430,6 @@ sigtrap_handler(int __attribute__((unused)) signal,
                 siginfo_t __attribute__((unused)) *info,
                 os_context_t *context)
 {
-#ifdef LISP_FEATURE_INT1_BREAKPOINTS
-    // ICEBP instruction = handle-pending-interrupt following pseudo-atomic
-    if (((unsigned char*)OS_CONTEXT_PC(context))[-1] == 0xF1)
-        return interrupt_handle_pending(context);
-#endif
-
     unsigned int trap;
 
     if (single_stepping) {
@@ -463,7 +457,15 @@ sigill_handler(int __attribute__((unused)) signal,
     unsigned char* pc = (void*)OS_CONTEXT_PC(context);
     if (UNALIGNED_LOAD16(pc) == UD2_INST) {
         OS_CONTEXT_PC(context) += 2;
+#ifdef LISP_FEATURE_UD2_BREAKPOINTS
         return sigtrap_handler(signal, siginfo, context);
+#else
+        /* UD2 ends pseudo-atomic sequences and has no trailing bytes that encode the
+         * reason for the trap. So the normal instruction stream is fully decodable by 'gdb'
+         * - which shows 0xCE as "(bad)" - or other tools, being devoid of arbitrary bytes
+         * that encode error metadata after the trapping instruction */
+        return interrupt_handle_pending(context);
+#endif
     }
     // Interrupt if overflow (INTO) raises SIGILL in 64-bit mode
     if (*(unsigned char *)pc == INTO_INST) {
@@ -853,7 +855,7 @@ os_vm_address_t coreparse_alloc_space(int space_id, int attr,
 {
     if (size == 0) return addr;
 
-    int extra_below = 0, extra_above = 0;
+    long extra_below = 0, extra_above = 0;
     extern int lisp_code_in_elf();
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
