@@ -585,7 +585,8 @@
      (floor a b))
    (values (integer -255 0) (integer -9 0) &optional)))
 
-(with-test (:name :logbitp-on-integers)
+(with-test (:name :logbitp-on-integers
+            :fails-on :arm)
   (assert (not (ctu:ir1-named-calls `(lambda (x)
                                        (logbitp 20 x))))))
 (with-test (:name :bt-negative-bit)
@@ -818,7 +819,8 @@
    ((400 2) (condition 'type-error))
    ((300 0) 303)))
 
-(with-test (:name :truncate-unknown-integer)
+(with-test (:name :truncate-unknown-integer
+            :fails-on :arm)
   (assert (not (ctu:ir1-named-calls `(lambda (x)
                                        (declare (integer x))
                                        (values (the (signed-byte 25) (floor x 2)))))))
@@ -838,7 +840,8 @@
                                        (declare (unsigned-byte x))
                                        (values (the (signed-byte 25) (ceiling x 2))))))))
 
-(with-test (:name :word-floor-ceiling)
+(with-test (:name :word-floor-ceiling
+            :fails-on :arm)
   (assert (not (ctu:ir1-named-calls `(lambda (x y)
                                        (declare (sb-vm:signed-word x y))
                                        (floor x y)))))
@@ -853,20 +856,22 @@
                                        (floor x y))))))
 
 (with-test (:name :fixnum-*-by-unknown)
-  (assert (not (ctu:ir1-named-calls `(lambda (x y)
-                                       (declare (integer x)
-                                                (fixnum y))
-                                       (the fixnum (* x y))))))
-  (assert (not (ctu:ir1-named-calls `(lambda (x y)
-                                       (declare (integer x)
-                                                (fixnum y))
-                                       (the fixnum (* y x))))))
+  (when (ctu:vop-existsp 'overflow*)
+    (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                         (declare (integer x)
+                                                  (fixnum y))
+                                         (the fixnum (* x y))))))
+    (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                         (declare (integer x)
+                                                  (fixnum y))
+                                         (the fixnum (* y x))))))
+    (assert (not (ctu:ir1-named-calls `(lambda (x y)
+                                         (declare (integer x y))
+                                         (the fixnum (* x y)))))))
   (assert (equal (ctu:ir1-named-calls `(lambda (x)
                                          (the fixnum (* x 2))))
                  '(sb-kernel:*-by-fixnum-to-fixnum)))
-  (assert (not (ctu:ir1-named-calls `(lambda (x y)
-                                       (declare (integer x y))
-                                       (the fixnum (* x y))))))
+
   (checked-compile-and-assert
       (:optimize :safe)
       `(lambda (a)
@@ -895,6 +900,119 @@
       (logxor x (1+ x))))
    (or null (integer 1))))
 
+(with-test (:name :logior--type)
+  (assert-type
+   (lambda (n)
+     (logior n (- n)))
+   (integer * 0))
+  (assert-type
+   (lambda (n)
+     (declare ((unsigned-byte 64) n))
+     (logior n (- n)))
+   (integer -9223372036854775808 0))
+  (assert-type
+   (lambda (n)
+     (declare ((signed-byte 64) n))
+     (logior n (- n)))
+   (integer -9223372036854775808 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -16 1024) n))
+     (logior n (- n)))
+   (integer -1024 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer #.(1+ (- (ash 1 63))) 0) n))
+     (logior n (- n)))
+   (integer -4611686018427387904 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -100 -10) n))
+     (logior n (- n)))
+   (integer -64 -1))
+  (assert-type
+   (lambda (n)
+     (declare ((integer 1) n))
+     (logior n (- n)))
+   (integer * -1)))
+
+(with-test (:name :logand--type)
+  (assert-type
+   (lambda (n)
+     (logand n (- n)))
+   (integer 0 *))
+  (assert-type
+   (lambda (n)
+     (declare ((unsigned-byte 64) n))
+     (logand n (- n)))
+   (integer 0 9223372036854775808))
+  (assert-type
+   (lambda (n)
+     (declare ((signed-byte 64) n))
+     (logand n (- n)))
+   (integer 0 9223372036854775808))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -16 1024) n))
+     (logand n (- n)))
+   (integer 0 1024))
+  (assert-type
+   (lambda (n)
+     (declare ((integer #.(1+ (- (ash 1 63))) 0) n))
+     (logand n (- n)))
+   (integer 0 4611686018427387904))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -100 -10) n))
+     (logand n (- n)))
+   (integer 1 64))
+  (assert-type
+   (lambda (n)
+     (declare ((integer 1) n))
+     (logand n (- n)))
+   (integer 1 *)))
+
+(with-test (:name :logxor--type)
+  (assert-type
+   (lambda (n)
+     (logxor n (- n)))
+   (integer * 0))
+  (assert-type
+   (lambda (n)
+     (declare ((unsigned-byte 64) n))
+     (logxor n (- n)))
+   (integer -18446744073709551616 0))
+  (assert-type
+   (lambda (n)
+     (declare ((signed-byte 64) n))
+     (logxor n (- n)))
+   (integer -18446744073709551616 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -16 1024) n))
+     (logxor n (- n)))
+   (integer -2048 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -1 0) n))
+     (logxor n (- n)))
+   (integer -2 0))
+  (assert-type
+   (lambda (n)
+     (declare ((integer -2 -1) n))
+     (logxor n (- n)))
+   (integer -4 -2))
+  (assert-type
+   (lambda (n)
+     (declare ((integer 1) n))
+     (logxor n (- n)))
+   (integer * -2))
+  (assert-type
+   (lambda (n)
+     (declare ((integer * -1) n))
+     (logxor n (- n)))
+   (integer * -2)))
+
 (with-test (:name :unsigned-byte-x-p)
   (checked-compile-and-assert
    ()
@@ -954,6 +1072,7 @@
    (((- (expt 2 160))) nil)
    (('a) nil)))
 
+
 (with-test (:name :signed-byte-x-p)
   (checked-compile-and-assert
       ()
@@ -1125,3 +1244,565 @@
      (declare ((integer 6671 6672) x))
      (values (ffloor -1333621864366 x)))
    (single-float -1.9991334e8 -1.9988338e8)))
+
+(with-test (:name :reduce-logior-type)
+  (assert-type
+   (lambda (a)
+     (reduce 'logior (the (simple-array (unsigned-byte 8)) a) :initial-value -7))
+   (integer -7 -1)))
+
+(with-test (:name :atanh-type)
+  (assert-type
+   (lambda (n)
+     (atanh (the (real -1 1) n)))
+   float))
+
+(with-test (:name :unknown-*-transform)
+  (checked-compile-and-assert
+   (:optimize :safe)
+   `(lambda (f n)
+      (declare (fixnum f)
+               ((unsigned-byte 65) n))
+      (the fixnum (* f n)))
+   ((0 (expt 2 64)) 0)
+   ((1 (expt 2 64)) (condition 'type-error))
+   ((2 2) 4))
+  (checked-compile-and-assert
+   (:optimize :safe)
+   `(lambda (f)
+      (declare (type fixnum f))
+      (the fixnum (* f 25152445588928537400)))
+   ((0) 0)
+   ((1) (condition 'type-error)))
+  (checked-compile-and-assert
+   (:optimize :safe)
+   `(lambda (a b)
+      (declare (integer a b))
+      (the fixnum (* a b)))
+   ((0 (expt 2 64)) 0)
+   ((1 (expt 2 64)) (condition 'type-error))
+   ((2 2) 4))
+  (checked-compile-and-assert
+   (:optimize :safe)
+   `(lambda (f)
+      (declare (integer f))
+      (the fixnum (* f 25152445588928537400)))
+   ((0) 0)
+   ((1) (condition 'type-error))))
+
+(with-test (:name :ash-negative-type)
+  (assert-type
+   (lambda (x y)
+     (declare ((real * 0) x))
+     (ash x y))
+   (integer * 0)))
+
+(with-test (:name :ash-overflow)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (declare (fixnum a))
+         (truly-the bit (ash a 90)))
+    ((0) 0)
+    ((1) 0)))
+
+(with-test (:name :range-unsigned-comparison)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (typep a '(integer 1 2147483647)))
+    ((-1) nil)
+    ((1) t)
+    ((2147483647) t)
+    ((2147483648) nil)
+    ((0.0) nil)))
+
+(with-test (:name :ash-right-two-word)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (logand 4611686018427387903
+                 (ash a -2)))
+    ((10) 2)
+    ((310805698068689180651640983330128115679) 585623305860541943)
+    ((-7514499718243589878) 2733061088866490434)))
+
+(with-test (:name :ash-left-add)
+  (checked-compile-and-assert
+      ()
+      `(lambda (p d)
+         (declare ((signed-byte 64) p)
+                  ((unsigned-byte 63) d))
+         (+ (ash p 63) d))
+    ((0 5) 5)
+    ((4 4) 36893488147419103236)
+    ((-5 5) -46116860184273879035))
+  (checked-compile-and-assert
+      ()
+      `(lambda (p d)
+         (declare ((signed-byte #.sb-vm:n-word-bits) p)
+                  ((unsigned-byte 8) d))
+         (+ (ash p 8) d))
+    ((0 5) 5)
+    ((4 4) 1028)
+    ((-5 4) -1276))
+  (checked-compile-and-assert
+      ()
+      `(lambda (p d)
+         (declare ((unsigned-byte 63) d))
+         (+ (ash p 63) d))
+    ((0 5) 5)
+    ((4 4) 36893488147419103236)
+    ((-5 5) -46116860184273879035))
+  (checked-compile-and-assert
+      ()
+      `(lambda (p d)
+         (declare ((unsigned-byte 8) d))
+         (+ (ash p 8) d))
+    ((0 5) 5)
+    ((4 4) 1028)
+    ((-5 4) -1276))
+  (checked-compile-and-assert
+      ()
+      `(lambda (p d)
+         (declare ((unsigned-byte 64) d))
+         (+ (ash p 64) d))
+    ((-1 17757265153539649207) -689478920169902409)
+    ((-1 0) -18446744073709551616)))
+
+(with-test (:name :remove-negate)
+  (flet ((test (form count)
+           (assert (= (count 'sb-kernel:%negate
+                             (ctu:ir1-named-calls form nil))
+                      count))))
+    (test `(lambda (x y)
+             (declare (integer x y))
+             (- (* x (- 5 y))))
+          0)
+    (test `(lambda (x y)
+             (- (truncate (- x) y)))
+          0)
+    (test `(lambda (x y)
+             (- (if x
+                    (- x)
+                    (- y))))
+          0)
+    (test `(lambda (x y)
+             (- (if x
+                    4
+                    (- y))))
+          0)
+    (test `(lambda (x y)
+             (declare (optimize (sb-c::float-accuracy 0)))
+             (- (* (/ x 4) (- y))))
+          0)
+    (test `(lambda (x y)
+             (- (truncate (- y) (* x 4))))
+          0)
+    (test `(lambda (a b)
+             (the integer (- (* a (- b)))))
+          0)
+    (test `(lambda (a b)
+             (+ (- (* a (- b))) 10))
+          0)
+    (test `(lambda (x a b)
+             (multiple-value-bind (a b) (if x
+                                            (funcall x)
+                                            (values 10 (- (* a (- b)))))
+               (values a
+                       (+ b 10))))
+          0)
+    (test `(lambda (x a b)
+             (multiple-value-bind (a b)
+                 (the (values t real)
+                      (if x
+                          (funcall x)
+                          (values 10 (- (* a (- b))))))
+               (values a
+                       (+ b 10))))
+          0)
+    (test `(lambda (x a b)
+             (multiple-value-bind (a b)
+                 (the (values t integer)
+                      (if x
+                          (funcall x)
+                          (values 10 (- (* a (- b))))))
+               (values a b)))
+          0)
+    (checked-compile-and-assert
+        ()
+        `(lambda (a)
+           (- (* (if a 1 2)
+                 (if a 3 4)
+                 -2)))
+      ((nil) 16)
+      ((t) 6))
+    (test `(lambda (a) (- (* a 0))) 1)
+    (test `(lambda (a) (- (* a 0.0))) 0)
+    (checked-compile-and-assert
+        ()
+        `(lambda (a)
+           (- (* a 0)))
+      ((10.0) -0.0)
+      ((10) 0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b c)
+           (- (* 4
+                 (if a
+                     (- b)
+                     (- c)))))
+      ((1 2 3) 8)
+      ((nil 2 3) 12))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a c d)
+           (declare ((integer 4 9) d))
+           (-
+            (if c
+                (+
+                 (if a 0 1)
+                 (if a 0 1)
+                 1)
+                (+ d 2))))
+      ((nil t 5) -3)
+      ((t nil 5) -7)
+      ((t t 4) -1)
+      ((nil nil 4) -6))
+    (checked-compile-and-assert
+     ()
+     `(lambda (e f j)
+        (-
+         (if f
+             (truncate
+              (if e 0 100)
+              (if j j -96))
+             1)))
+     ((nil 2 3) -33)
+     ((nil nil 5) -1)
+     ((nil t nil) 1))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b c)
+           (- (if a -3 (* (truncate 4 b) c))))
+      ((nil 2 3) -6)
+      ((t 4 5) 3))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b)
+           (- (* (* a 3) b)))
+      ((0 4.0) -0.0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b)
+           (- (* a (- b))))
+      ((4.0 0) -0.0))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b)
+           (declare (integer b))
+           (- (- (+ a 5)) b))
+      ((-5.0 0) -0.0))))
+
+(with-test (:name :remove-negate.2)
+  (flet ((test (form count)
+           (assert (= (count 'sb-kernel:%negate
+                             (ctu:ir1-named-calls form nil))
+                      count))))
+    (test `(lambda (x y)
+             (declare (optimize (sb-c::float-accuracy 0)))
+             (* (- x) (- y)))
+          0)
+    (test `(lambda (x y)
+             (> (* (- x) (- y)) 10))
+          0)
+    (test `(lambda (a b d)
+             (declare (optimize (sb-c::float-accuracy 0)))
+             (- d (* a (- b))))
+          0)
+    (test `(lambda (x y)
+             (abs (* (- x) y)))
+          0)
+    (test `(lambda (x y)
+             (abs (if x
+                      (- x)
+                      y)))
+          0)
+    (test `(lambda (x)
+             (lognot (- x)))
+          0)
+    (test `(lambda (x)
+             (- (lognot x)))
+          0)))
+
+(with-test (:name :abs-match)
+  (flet ((test (form count)
+           (assert (= (count 'abs
+                             (ctu:ir1-named-calls form nil))
+                      count))))
+    (test `(lambda (x y)
+             (declare (real x))
+             (abs (* (abs x) y)))
+          1)
+    (test `(lambda (x y)
+             (declare (real x y))
+             (/ (abs y) (abs x)))
+          1)
+    (test `(lambda (x)
+             (declare (real x))
+             (abs (* x x)))
+          0)
+    (test `(lambda (x y)
+             (declare (real x))
+             (abs (/ (* (abs x) y) 4)))
+          1)
+    (test `(lambda (a b c)
+             (abs
+              (if a
+                  (abs b)
+                  c)))
+          1)
+    (test `(lambda (a)
+             (declare (real a))
+             (= (abs a) 10))
+          0)))
+
+(with-test (:name :*-by-zero-type)
+  (assert-type
+   (lambda (d)
+     (* (the (double-float -10d0 10d0) d) 0.0d0))
+   (double-float 0d0 0d0))
+  (assert-type
+   (lambda (d)
+     (* (the (double-float * 10d0) d) 0.0d0))
+   (double-float * 0d0))
+  (assert-type
+   (lambda (x y)
+     (* (the (single-float 0.0 0.0) x)
+        (the (single-float -5.0 5.0) y)))
+   (single-float 0.0 0.0))
+  (assert-type
+   (lambda (x)
+     (* (the (integer -1 0) x) 4.0))
+   (single-float -4.0 0.0))
+  (assert-type
+   (lambda (x)
+     (* -0.0 (the (single-float 0.0 0.0) x)))
+   (single-float 0.0 0.0))
+  (assert-type
+   (lambda (x)
+     (abs (the (or (single-float -2.0 -1.0) (member -0.0)) x)))
+   (or (single-float 1.0 2.0) (member 0.0))))
+
+(with-test (:name :logior-signed-unsigned=>integer)
+  (checked-compile-and-assert
+   ()
+   `(lambda (s u)
+      (declare ((unsigned-byte 64) u))
+      (logior (the fixnum s) u))
+   ((-54327132 24) -54327108)
+   ((54327132 15028999435905310454) 15028999435923161086)))
+
+(with-test (:name :dpb-bit-overshoot)
+  (checked-compile-and-assert
+   ()
+   `(lambda (p)
+      (dpb 0 (byte 1 (the (mod 95) p)) #xFFFFFFFFFFF))
+   ((0) #xFFFFFFFFFFE)
+   ((63) #xFFFFFFFFFFF)
+   ((64) #xFFFFFFFFFFF)
+   ((79) #xFFFFFFFFFFF))
+  (checked-compile-and-assert
+   ()
+   `(lambda (p)
+      (dpb 0 (byte 1 (the (mod 95) p)) #xFFFFFFFFFFFFFFFF))
+   ((0) #xFFFFFFFFFFFFFFFE)
+   ((63) #x7FFFFFFFFFFFFFFF)
+   ((64) #xFFFFFFFFFFFFFFFF)
+   ((79) #xFFFFFFFFFFFFFFFF))
+  (checked-compile-and-assert
+   ()
+   `(lambda (p)
+      (dpb 1 (byte 1 (the (mod 95) p)) -10))
+   ((0) -9)
+   ((79) -10)))
+
+(with-test (:name :ctz)
+  (when (ctu:vop-existsp 'sb-kernel:count-trailing-zeros)
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare ((unsigned-byte 64) n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil)))
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare ((signed-byte 64) n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil)))
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare (fixnum n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil)))
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare ((signed-byte 32) n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil)))
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare ((unsigned-byte 63) n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil)))
+    (assert (find 'sb-kernel:count-trailing-zeros
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare ((unsigned-byte 62) n))
+                                          (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+                                       nil))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare ((unsigned-byte 64) n))
+         (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+    ((0) 64)
+    ((1) 0)
+    ((2) 1)
+    (((ash 1 63)) 63))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare ((signed-byte 64) n))
+         (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+    ((0) 64)
+    ((1) 0)
+    ((-1) 0)
+    ((2) 1)
+    ((-2) 1)
+    (((ash 1 62)) 62)
+    (((ash -1 63)) 63))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare (fixnum n))
+         (integer-length (ldb (byte 64 0) (lognor n (- n)))))
+    ((0) 64)
+    ((1) 0)
+    ((-1) 0)
+    ((2) 1)
+    ((-2) 1)
+    (((ash 1 (- sb-vm:n-fixnum-bits 2))) (- sb-vm:n-fixnum-bits 2))
+    (((ash -1 (- sb-vm:n-fixnum-bits 1))) (- sb-vm:n-fixnum-bits 1))))
+
+(with-test (:name :ash-left-add)
+  (when (ctu:vop-existsp 'sb-kernel:ash-left-add)
+    (assert (find 'sb-kernel:ash-left-add
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare (sb-vm:signed-word n))
+                                          (+ (ash n 8) 8))
+                                       nil)))
+    (assert (find 'sb-kernel:ash-left-add
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (declare (sb-vm:word n))
+                                          (+ (ash n 8) 8))
+                                       nil)))
+    (assert (find 'sb-kernel:ash-left-add
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (+ (ash n 8) 8))))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare ((unsigned-byte 64) n))
+         (+ (ash n 8) 8))
+    ((0) 8)
+    ((1) (+ (ash 1 8) 8))
+    ((#xFFFFFFFFFFFFFFF) (+ (ash #xFFFFFFFFFFFFFFF 8) 8))
+    ((#xFFFFFFFFFFFFFFFF) (+ (ash #xFFFFFFFFFFFFFFFF 8) 8)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare ((signed-byte 64) n))
+         (+ (ash n 8) 8))
+    ((0) 8)
+    ((1) (+ (ash 1 8) 8))
+    ((-1) (+ (ash -1 8) 8))
+    ((#xFFFFFFFFFFFFFFF) (+ (ash #xFFFFFFFFFFFFFFF 8) 8))
+    ((#x-FFFFFFFFFFFFFFF) (+ (ash #x-FFFFFFFFFFFFFFF 8) 8))
+    ((#x7FFFFFFFFFFFFFFF) (+ (ash #x7FFFFFFFFFFFFFFF 8) 8))
+    (((- (expt 2 63))) (+ (ash (- (expt 2 63)) 8) 8)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (+ (ash n 8) 8))
+    ((0) 8)
+    ((1) (+ (ash 1 8) 8))
+    ((-1) (+ (ash -1 8) 8))
+    ((#xFFFFFFFFFFFFFFF) (+ (ash #xFFFFFFFFFFFFFFF 8) 8))
+    ((#x-FFFFFFFFFFFFFFF) (+ (ash #x-FFFFFFFFFFFFFFF 8) 8))
+    ((#xFFFFFFFFFFFFFFFF) (+ (ash #xFFFFFFFFFFFFFFFF 8) 8))
+    ((#x7FFFFFFFFFFFFFFF) (+ (ash #x7FFFFFFFFFFFFFFF 8) 8))
+    (((- (expt 2 63))) (+ (ash (- (expt 2 63)) 8) 8))))
+
+(with-test (:name :ash-right-two-words)
+  (when (ctu:vop-existsp 'sb-kernel:ash-right-two-words)
+    (assert (find 'sb-kernel:ash-right-two-words
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (logand (ash n -8) most-positive-fixnum))
+                                       nil)))
+    (assert (find 'sb-kernel:ash-right-two-words
+                  (ctu:ir1-named-calls `(lambda (n s)
+                                          (declare ((integer -64 -1) s))
+                                          (logand (ash n s) most-positive-fixnum))
+                                       nil)))
+    (assert (find 'sb-kernel:ash-right-two-words
+                  (ctu:ir1-named-calls `(lambda (n)
+                                          (logand (ash n -8) most-positive-word))
+                                       nil)))
+    (assert (find 'sb-kernel:ash-right-two-words
+                  (ctu:ir1-named-calls `(lambda (n s)
+                                          (declare ((integer -64 -1) s))
+                                          (logand (ash n s) most-positive-word))
+                                       nil))))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (logand (ash n -8) most-positive-fixnum))
+    ((0) 0)
+    (((ash 1 10)) (ash 1 2))
+    (((ash -1 10)) (logand (ash -1 2) most-positive-fixnum))
+    ((20473335672995943448) (logand 79973967472640404 most-positive-fixnum)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n s)
+         (declare ((integer -64 -1) s))
+         (logand (ash n s) most-positive-fixnum))
+    ((0 -10) 0)
+    (((ash 1 12) -10) (ash 1 2))
+    (((ash -1 12) -10) (logand (ash -1 2) most-positive-fixnum))
+    ((325822690411775662515259164035444996112 -64) (logand 17662883439475955428 most-positive-fixnum)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (logand (ash n -8) most-positive-word))
+    ((0) 0)
+    (((ash 1 10)) (ash 1 2))
+    (((ash -1 10)) (logand (ash -1 2) most-positive-word))
+    ((20473335672995943448) (logand 79973967472640404 most-positive-word)))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n s)
+         (declare ((integer -64 -1) s))
+         (logand (ash n s) most-positive-word))
+    ((0 -10) 0)
+    (((ash 1 12) -10) (ash 1 2))
+    (((ash -1 12) -10) (logand (ash -1 2) most-positive-word))
+    ((325822690411775662515259164035444996112 -64) (logand 17662883439475955428 most-positive-word))))
+
+
+(with-test (:name :mask-signed-field-word-move)
+  (checked-compile-and-assert
+      ()
+      `(lambda (n)
+         (declare (word n))
+         (logand (sb-c::mask-signed-field sb-vm:n-fixnum-bits n) most-positive-word))
+    (((* most-positive-fixnum 2))
+     (logand -2 most-positive-word))))
