@@ -11,6 +11,8 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
+(setq *random-state* (make-random-state t))
+
 ;;; As reported by Paul Dietz from his ansi-test suite for gcl, REMOVE
 ;;; malfunctioned when given :START, :END and :FROM-END arguments.
 ;;; Make sure it doesn't happen again.
@@ -237,7 +239,7 @@
              (case safety
                (0 (= speed 0))
                (t t)))
-           (extra-safe (&key speed safety &allow-other-keys)
+           (extra-safe (&key safety &allow-other-keys)
              (>= safety 2))
            (test (type expr &key (filter #'safe))
              (checked-compile-and-assert
